<?php

namespace SGalinski\SgJobs\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgJobs\Domain\Model\JobApplication;
use SGalinski\SgJobs\Property\TypeConverter\UploadedFileReferenceConverter;
use SGalinski\SgJobs\Service\FrontendFilterService;
use SGalinski\SgMail\Service\MailTemplateService;
use SJBR\StaticInfoTables\Domain\Repository\CountryRepository;
use TYPO3\CMS\Core\Resource\ResourceFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;

/**
 * The joblist plugin controller
 */
class JoblistController extends ActionController {
	/**
	 * @var \SGalinski\SgJobs\Domain\Repository\CompanyRepository
	 * @inject
	 */
	private $companyRepository;

	/**
	 * @var \SGalinski\SgJobs\Domain\Repository\JobRepository
	 * @inject
	 */
	private $jobRepository;

	/**
	 * Show all job offers and options to manage them
	 *
	 * @param array $filters
	 * @return void
	 * @throws \InvalidArgumentException
	 */
	public function indexAction(array $filters = []) {
		if ($filters) {
			$this->view->assign('selectedCountry', $filters['filterCountry']);
			$this->view->assign('selectedCompany', $filters['filterCompany']);
			$this->view->assign('selectedLocation', $filters['filterLocation']);
			$this->view->assign('selectedArea', $filters['filterArea']);
			$this->view->assign('selectedFunction', $filters['filterFunction']);
		}

		$storagePid = (int) $this->configurationManager->getConfiguration(
			ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK
		)['persistence']['storagePid'];

		$this->assignFilterValues($storagePid);
		$this->view->assign('recordPageId', $storagePid);

		// pagination logic
		$jobLimit = (int) $this->settings['jobLimit'];
		$offset = 0;
		$currentPageBrowserPage = (int) GeneralUtility::_GP('tx_sgjobs_pagebrowser')['currentPage'];
		if ($currentPageBrowserPage && $jobLimit) {
			$offset = $currentPageBrowserPage * $jobLimit;
		}

		// get all jobs for the current page
		$jobs = FrontendFilterService::getJobs($filters, $storagePid, $jobLimit, $offset);

		// get all jobs for the current page
		$jobsCount = \count(FrontendFilterService::getJobs($filters, $storagePid));
		$numberOfPages = ($jobLimit <= 0 ? 0 : ceil($jobsCount / $jobLimit));

		$this->view->assign('jobs', $jobs);
		$this->view->assign('limit', $jobLimit);
		$this->view->assign('numberOfPages', $numberOfPages);
	}

	/**
	 * Renders the application form with an optional job
	 *
	 * @param JobApplication $applyData
	 * @param string $error
	 * @throws \InvalidArgumentException
	 */
	public function applyFormAction(JobApplication $applyData = NULL, $error = NULL) {
//		$uploadedFiles = $this->getExistingApplicationFiles($uniqueFolderName);
//		if (\count($uploadedFiles['coverLetter']) <= 0 && \count($uploadedFiles['cv']) <= 0
//			&& \count($uploadedFiles['certificate']) <= 0
//		) {
//			$this->forward('applyForm', NULL, NULL, ['error' => 'Bitte laden Sie alle benötigten Dateien hoch!']);
//		}

//		$this->request->setArgument('uploadedFiles', $uploadedFiles);

		if ($error !== NULL && $error !== '') {
			$this->view->assign('internalError', $error);
		}

		$jobId = $this->request->getArguments()['jobData']['uid'];
		if (!empty($jobId)) {
			$jobData = $this->jobRepository->findByUid($jobId);
			$this->view->assign('job', $jobData);
		}

		// display country options
		$countryRepository = $this->objectManager->get(CountryRepository::class);
		$countries = $countryRepository->findAllOrderedBy('shortNameEn')->toArray();
		$this->view->assign('countries', $countries);
		$this->view->assign('sysLanguageUid', $GLOBALS['TSFE']->sys_language_uid);

		$allowedMimeTypes = $this->settings['allowedMimeTypes'];
		$this->view->assign('allowedMimeTypes', $allowedMimeTypes);
		$allowedFileExtensions = $this->settings['allowedFileExtensions'];
		$this->view->assign('allowedFileExtensions', $allowedFileExtensions);
		$this->view->assign('applyData', $applyData);
	}

	/**
	 * Pre-apply action setup, configures model-property mapping and handles file upload
	 *
	 * @return void
	 * @throws \TYPO3\CMS\Core\Resource\Exception\InsufficientFolderWritePermissionsException
	 * @throws \TYPO3\CMS\Core\Resource\Exception\InsufficientFolderAccessPermissionsException
	 * @throws \TYPO3\CMS\Core\Resource\Exception\ExistingTargetFolderException
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\InvalidArgumentNameException
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException
	 * @throws \Exception
	 */
	protected function initializeApplyAction() {
		if ($this->request->hasArgument('folderName')) {
			$uniqueFolderName = $this->request->getArgument('folderName');
		} else {
			$uniqueFolderName = uniqid('sgjobs-', TRUE);
			$resourceFactory = $this->objectManager->get(ResourceFactory::class);
			$storage = $resourceFactory->getStorageObject(1);
			$storage->createFolder('/JobApplication/' . $uniqueFolderName);
		}
		$this->request->setArgument('folderName', $uniqueFolderName);

		$propertyMappingConfiguration = $this->arguments->getArgument('applyData')->getPropertyMappingConfiguration();

		$typeConverter1 = $this->objectManager->get(UploadedFileReferenceConverter::class);
		$typeConverter1->setAllowedFileExtensions($this->settings['allowedFileExtensions']);
		$typeConverter1->setUploadFolder('1:/JobApplication/' . $uniqueFolderName);
		$typeConverter1->setTargetUploadFileName('coverLetter');
		$propertyMappingConfiguration->forProperty('coverLetter')->setTypeConverter($typeConverter1);

		$typeConverter2 = $this->objectManager->get(UploadedFileReferenceConverter::class);
		$typeConverter2->setAllowedFileExtensions($this->settings['allowedFileExtensions']);
		$typeConverter2->setUploadFolder('1:/JobApplication/' . $uniqueFolderName);
		$typeConverter2->setTargetUploadFileName('cv');
		$propertyMappingConfiguration->forProperty('cv')->setTypeConverter($typeConverter2);

		$typeConverter3 = $this->objectManager->get(UploadedFileReferenceConverter::class);
		$typeConverter3->setAllowedFileExtensions($this->settings['allowedFileExtensions']);
		$typeConverter3->setUploadFolder('1:/JobApplication/' . $uniqueFolderName);
		$typeConverter3->setTargetUploadFileName('certificate');
		$propertyMappingConfiguration->forProperty('certificate')->setTypeConverter($typeConverter3);
	}

	/**
	 * Moves the application files from temporary to permanent storage
	 *
	 * @param JobApplication $applicationData
	 * @param string $folderName
	 * @return void
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\StopActionException
	 * @throws \RuntimeException
	 * @throws \InvalidArgumentException
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\UnsupportedRequestTypeException
	 */
	private function submitApplicationFiles(JobApplication $applicationData, $folderName) {
		$resourceFactory = $this->objectManager->get(ResourceFactory::class);
		$newName = date('Ymd-His') . '_' . $applicationData->getFirstName();
		$storage = $resourceFactory->getStorageObject(1);
		$applicationFilePath = str_replace('/', '', $storage->getConfiguration()['basePath']) . $folderName .
			'/' . $newName . '.csv';
		$this->writeApplicationFile($applicationData, $applicationFilePath);
	}

	/**
	 * Saves the application send by the applyFormAction
	 *
	 * @param JobApplication $applyData
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\StopActionException
	 * @throws \InvalidArgumentException
	 */
	public function applyAction(JobApplication $applyData) {
		try {
			$applyData->setPid($GLOBALS['TSFE']->id);
			$folderName = $this->request->getArgument('folderName');
			$this->submitApplicationFiles($applyData, $folderName);

			/** @noinspection PhpMethodParametersCountMismatchInspection */
			$mailService = $this->objectManager->get(
				MailTemplateService::class, 'application_mail', 'sg_jobs',
				$this->getApplicationMailMarkers($applyData)
			);
			$mailService->setMarkers(['application' => $applyData]);
			$mailService->setIgnoreMailQueue(TRUE);
			$mailService->sendEmail();

			// redirect to the given page id from the flexform
			$redirectPageId = (int) $this->configurationManager->getConfiguration(
				ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK
			)['settings']['redirectPage'];

			$uriBuilder = $this->uriBuilder;
			$uri = $uriBuilder->reset()->setTargetPageUid($redirectPageId)->build();
			$this->redirectToUri($uri);

		} catch (\Exception $exception) {
			$this->forward('applyForm', NULL, NULL, ['error' => $exception->getMessage()]);
		}
	}

	/**
	 * Assign filter values
	 *
	 * @param int $rootPageId
	 */
	private function assignFilterValues($rootPageId) {
		$countries = $this->companyRepository->getAllCountries($rootPageId);
		$this->view->assign('countries', $countries);

		$cities = $this->companyRepository->getAllCities($rootPageId);
		$this->view->assign('cities', $cities);

		$companies = $this->companyRepository->getAllCompanyNames($rootPageId);
		$this->view->assign('companies', $companies);

		$areas = $this->jobRepository->getAllAreas($rootPageId);
		$this->view->assign('areas', $areas);

		$functions = $this->jobRepository->getAllFunctions($rootPageId);
		$this->view->assign('functions', $functions);
	}

	/**
	 * Returns the application mail markers
	 *
	 * @param JobApplication $applyData
	 * @return array
	 */
	private function getApplicationMailMarkers($applyData): array {
		return [
			'salutation' => $applyData->getGender(),
			'firstname' => $applyData->getFirstName(),
			'lastname' => $applyData->getLastName(),
			'street' => $applyData->getStreet(),
			'city' => $applyData->getCity(),
			'country' => $applyData->getCountry(),
			'phone' => $applyData->getPhone(),
			'mobile' => $applyData->getMobile(),
			'email' => $applyData->getEmail(),
			'message' => $applyData->getFirstName()
		];
	}

	/**
	 * Writes the application files
	 *
	 * @param JobApplication $data
	 * @param string $filePath
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\UnsupportedRequestTypeException
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\StopActionException
	 * @throws \InvalidArgumentException
	 * @throws \RuntimeException
	 */
	private function writeApplicationFile(JobApplication $data, $filePath) {
		$dataToInsertArr = [
			$data->getJobId(),
			$data->getFirstName(),
			$data->getLastName(),
			$data->getGender(),
			$data->getCountry(),
			$data->getBirthDate(),
			$data->getEducation(),
			$data->getStreet(),
			$data->getZip(),
			$data->getCity(),
			$data->getNationality(),
			$data->getPhone(),
			$data->getEmail(),
			$data->getCoverLetter()->getOriginalResource()->getPublicUrl(),
			$data->getCv()->getOriginalResource()->getPublicUrl(),
			$data->getCertificate()->getOriginalResource()->getPublicUrl(),
		];

		try {
			$file = fopen($filePath, 'wb+');
			fputcsv($file, $dataToInsertArr);
			fclose($file);
		} catch (\RuntimeException $exception) {
			$this->redirect('applyForm', NULL, NULL, ['error' => $exception->getMessage()]);
		}
	}
}
